package asd.td1;

/**
 * La classe Entier pour les exercices 2  6 de la feuille 1.
 * @author  2002 Marc Gatano
 */
public class Entier {

    /**
     * Construit un objet Entier vide pour appliquer les
     * mthodes qui suivent.
     */
    public Entier() {
    }

    // Exercice 2 /////////////////////////////////////////////////////////

    /**
     * Imprime tous les diviseurs de n.
     * Prcondition : n est positif ou nul.
     * Postrelation : les diviseurs de n ont t imprims.
     * Complexit : O(n)
     */
    public void diviseurs(int n) {
	System.out.print("Les diviseurs de " + n + " sont : ");
	
	// INVARIANT : les diviseurs de n strictement infrieurs  d
	// ont t imprims
	// VARIANT : d crot et est major par (n + 1)
	//
        for ( int d = 1; d <= n; d++ ) {
            if ( n % d == 0 ) {
                System.out.print(" " + d);
            }
        }
        System.out.println();
    }

    // Exercice 3 /////////////////////////////////////////////////////////

    /**
     * Imprime les facteurs premiers de n.
     * Prcondition : n > 1.
     * Postrelation : les facteurs premiers de n sont imprims.
     * Complexit : O(n)
     */
    public void decomposition(int n) {
	int x = n;
	int d = 2;

	System.out.print("Les facteurs premiers de " + x + " sont : ");

	// INVARIANT : soient p1,p2,..,pK premiers tels que
	// n = p1 * p2 * .. * pK * x :
	//  - p1, p2, .., pK ont t imprim
	//  - les facteurs de x sont >= d
	// VARIANT : x dcrot et est minor par 1
	//
	while ( x > 1 ) {

	    // INVARIANT : pour tout d' < d, d' ne divise pas x et d <= x
	    // VARIANT : d crot et est major par x,
	    //           et quand d = x, x % d == 0
	    //
	    while ( x % d != 0 ) {
		d++;
	    }
	    
	    System.out.print(" " + d);
	    x = x / d;
        }
        System.out.println();
    }

    // Exercice 4 /////////////////////////////////////////////////////////

    /**
     * Teste si n est un palindrome.
     * Prcondition : n >= 0.
     * Valeur rendue : true si n est un palindrome.
     * Complexit : THETA(log[10](n))
     * o log[a](b) est le logarithme de b en base a
     */
    public boolean palindrome(int n) {
        int ab = n;
        int ba = 0;

	// INVARIANT : si C(x) est le nombre chiffres de x (avec C(0) = 0),
	// R(x) le "renvers" de x, alors : n = ab * 10^C(ba) + R(ba)
	// VARIANT   : ab dcrot et est minor par 0
	//
        while ( ab > 0 ) {
            ba = 10 * ba + (ab % 10);
            ab = ab / 10;
        }
        return n == ba;
    }

    // Exercice 5 /////////////////////////////////////////////////////////

    /**
     * Teste si n est un mirroir.
     * Prcondition : n >= 0.
     * Valeur rendue : true si n est un mirroir.
     * Complexit : O(log[10](n))
     * o log[a](b) est le logarithme de b en base a
     */
    public boolean mirroir(int n) {
        int ab = n;
        int ba = 0;

	// INVARIANT : si C(x) est le nombre chiffres de x (C(0) = 0),
	// R(x) le "renvers" de x, alors : n = ab * 10^C(ba) + R(ba)
	// VARIANT   : ab - ba dcrot et est minor par 0
	//
        while ( ab > ba ) {
            ba = 10 * ba + (ab % 10);
            ab = ab / 10;
        }

        return ab != 0 && ab == ba;
    }

    // Exercice 6 /////////////////////////////////////////////////////////

    /**
     * Imprime les n premiers nombres premiers.
     * Prcondition : n >= 0.
     * Postrelation : les n premiers nombres premiers.
     * Complexit : OMEGA(n^2)
     */
    public void premiers(int n) {
        int[] premiers = new int[n];
        premiers[0] = 2;
        
        int k = 1;
        int p = 3;
        
	// INVARIANT : 
	// - premiers[0..k-1] est la suite croissante des k premiers premiers
	// - pour tout q, premiers[k-1] < q < p, q n'est pas premier
	// - p est impair
	// VARIANT : k crot et est born par premiers.length ou p crot et
	//           il existe forcment un nombre premier suprieur  p
	//
        while ( k < premiers.length ) {

            int j = 0;

	    // INVARIANT : pour tout q dans premiers[0..j-1], p n'est pas
	    // divisible par q
	    // VARIANT : j crot et est born par k
	    //
            while ( j < k && ( p % premiers[j] != 0 ) ) {
		j++;
	    }
	    
            if ( j == k ) {
                premiers[k] = p;
                k++;
            }
            p += 2;
        }

	System.out.print("les " + n + " premiers nombres premiers sont");

        for ( int j = 0; j < premiers.length; j++ ) {
            System.out.print(" " + premiers[j]);
        }
        System.out.println();
    }
}
